import tradermade as tm
from loguru import logger

from ._client import Client


class TradermadeHandler(Client):
    """
    Tradermade API client
    documentation: https://github.com/tradermade/Python-SDK
    or https://tradermade.com/tutorials/python-sdk-for-forex-data


    """

    def __init__(self, **kwargs):
        """
        Initialize the object with the given keyword arguments.

        :param kwargs: keyword arguments
        :return: None
        """

        super().__init__(**kwargs)
        if self.enabled:
            self.client = "tradermade"
            logger.info("Initializing Tradermade")
            tm.set_rest_api_key(self.api_key)

    async def get_news(self):
        """ """
        # TODO
        # return tm.live(currency="EURUSD,GBPUSD", fields=["bid", "mid", "ask"])
        data = tm.live(currency="EURUSD,GBPUSD", fields=["bid", "mid", "ask"])

        try:
            # Convert the data to a string format
            result = []
            for currency, values in data.items():
                result.append(f"{currency}:")
                result.extend(f"  {field}: {value}" for field, value in values.items())
            return "\n".join(result)
        except Exception as e:
            return f"Error fetching currency data: {str(e)}"



import finnhub

from ._client import Client


class FinnhubHandler(Client):
    """
    Finnhub API client

    documentation: https://finnhub.io/docs/api

    """

    def __init__(self, **kwargs):
        """
        Initialize the object with the given keyword arguments.

        :param kwargs: keyword arguments
        :return: None
        """

        super().__init__(**kwargs)
        if self.enabled:
            self.client = finnhub.Client(api_key=self.api_key)
            self.websocket_url = f"wss://ws.finnhub.io?token={self.api_key}"

    async def get_news(self):
        """
        Asynchronously retrieves news articles from the Finnhub API
        based on the specified category and API key.

        :return: A string containing HTML formatted news summaries
        linked to their respective URLs.
        Returns None if an error occurs while retrieving the news.
        """

        news_data = self.client.general_news(self.api_category, min_id=0)
        # Create HTML formatted string for each news item
        news_summary_html = (
            f"<a href='{item['url']}' target='_blank'>{item['headline']}</a>"
            f"<br/><p>{item['summary']}</p>"
            for item in news_data
            if "headline" in item and "url" in item and "summary" in item
        )

        return "<br/>".join(news_summary_html)

    # async def stream(self):
    #     """
    #     Asynchronously streams data from the source
    #     using the configured settings.

    #     Returns:
    #         str: A string containing the concatenated results
    #          of the retrieved data sources.
    #     """
    #     # async with websockets.connect(self.websocket_url) as websocket:
    #     #     async for message in websocket:
    #     #         yield message

    #     with websockets.connect(self.websocket_url) as websocket:
    #         message = websocket.recv()
    #         logger.info(f"Received: {message}")
    #         yield message



import websockets
from loguru import logger


class Client:

    def __init__(self, **kwargs):
        """
        Initializes the Client object
        with the given keyword arguments.

        :param kwargs: Variable length keyword arguments.
        :type kwargs: dict

        :return: None
        """

        logger.info("Initializing Client")
        get = kwargs.get
        self.name = get("name", None)
        self.enabled = get("enabled", True)
        self.library = get("library", None)
        self.instrument = get("instrument", None)
        self.format = get("format", None)
        self.url = get("url", None)
        self.url_element = get("url_element", None)
        self.api_key = get("api_key", None)
        self.api_category = get("api_category", None)
        self.ticker = get("ticker", "AAPL")
        self.stream = get("stream", False)
        self.websocket_url = get("websocket_url", None)
        self.client = None

    async def fetch(self):
        """
        Asynchronously fetches data from the source
        using the configured settings.

        Returns:
            str: A string containing the concatenated results
             of the retrieved data sources.
        """

    async def monitor(self):
        """
        Asynchronously monitors the system and retrieves
        various data sources based on the configured settings.
        Cover Events, Feed, and Signal.

        Returns:
            str: A string containing the concatenated results
             of the retrieved data sources.
        """

    async def get_news(self):
        """
        Asynchronously retrieves the latest news
        from various sources based on the configured
        settings.

        Returns:
            str: A string containing the concatenated
             results of the retrieved news sources.
        """

    # async def stream(self):
    #     """
    #     Asynchronously streams data from the source
    #     using the configured settings.

    #     Returns:
    #         str: A string containing the concatenated results
    #          of the retrieved data sources.
    #     """
    async def stream(self):
        """
        Asynchronously streams data from the source
        using the configured settings.

        Returns:
            str: A string containing the concatenated results
             of the retrieved data sources.
        """

        if self.websocket_url is None:
            return
        with websockets.connect(self.websocket_url) as websocket:
            message = websocket.recv()
            logger.info(f"Received: {message}")
            yield message



# from loguru import logger
# from polygon import RESTClient

# from ._client import Client


# class PolygonHandler(Client):
#     """
#     Polygon.io client
#     docs: https://github.com/polygon-io/client-python


#     """

#     def __init__(self, **kwargs):
#         """
#         Initialize the object with the given keyword arguments.

#         :param kwargs: keyword arguments
#         :return: None
#         """

#         super().__init__(**kwargs)
#         if self.enabled:
#             self.client = RESTClient(api_key=self.api_key)
#             logger.info("Initializing Polygon.io")

#     async def get_news(self):
#         """ """
#         return self.client.list_ticker_news(self.ticker, order="desc", limit=1)



from talkytrend.handler.alphavantage import AlphavantageHandler
from talkytrend.handler.calendar import CalendarHandler
from talkytrend.handler.feed import FeedHandler
from talkytrend.handler.finnhub import FinnhubHandler
from talkytrend.handler.forexnewsapi import ForexnewsapiHandler
from talkytrend.handler.livetv import LivetvHandler
from talkytrend.handler.marketaux import MarketauxHandler
from talkytrend.handler.scraper import ScraperHandler

# from talkytrend.handler.tradermade import TradermadeHandler
from talkytrend.handler.tradingeconomics import TradingeconomicsHandler
from talkytrend.handler.tradingview import TradingviewHandler
from talkytrend.handler.twelvedata import TwelvedataHandler

# from talkytrend.handler.websocket import WebsocketHandler
from talkytrend.handler.yfinance import YfinanceHandler

__all__ = [
    "TradingviewHandler",
    "YfinanceHandler",
    "CalendarHandler",
    "FeedHandler",
    "ScraperHandler",
    "LivetvHandler",
    "FinnhubHandler",
    "AlphavantageHandler",
    "MarketauxHandler",
    "TradingeconomicsHandler",
    "TwelvedataHandler",
    "ForexnewsapiHandler",
    # "WebsocketHandler",
    # "TradermadeHandler",
]



import aiohttp
from loguru import logger

from ._client import Client


class MarketauxHandler(Client):
    """
    Marketaux client
    documentation: https://www.marketaux.com/documentation

    """

    def __init__(self, **kwargs):
        """
        Initialize the object with the given keyword arguments.

        :param kwargs: keyword arguments
        :return: None
        """

        super().__init__(**kwargs)
        if self.enabled:
            self.client = "Marketaux"
            logger.info("Initializing Marketaux with self.url={}", self.url)

    async def get_news(self):
        """
        Asynchronously retrieves news articles from the Marketaux endpoint
        based on the specified url.

        :return: A string containing HTML formatted news summaries
        linked to their respective URLs.
        Returns None if an error occurs while retrieving the news.
        """
        async with aiohttp.ClientSession() as session:
            async with session.get(self.url, timeout=10) as response:
                logger.debug("Fetching events from {}", self.url)
                response.raise_for_status()
                data = await response.json()

                news_articles = []
                for article in data["data"]:
                    url = article["url"]
                    title = article["title"]
                    # sentiment = article["entities"][0]["sentiment_score"]
                    article_summary = f"<a href='{url}'>{title}</a><br>"

                    news_articles.append(article_summary)

                return "<br><br>".join(news_articles)



from loguru import logger
from twelvedata import TDClient

from ._client import Client


class TwelvedataHandler(Client):
    """
    Twelvedata client
    docs: https://github.com/twelvedata/twelvedata-python


    """

    def __init__(self, **kwargs):
        """
        Initialize the object with the given keyword arguments.

        :param kwargs: keyword arguments
        :return: None
        """

        super().__init__(**kwargs)
        if self.enabled:
            self.client = TDClient(apikey=self.api_key)
            logger.info("Initializing Twelvedata")

    async def get_news(self):
        """ """
        # TODO



import yfinance as yf
from loguru import logger

from ._client import Client


class YfinanceHandler(Client):
    """

    YfinanceHandler


    """

    def __init__(self, **kwargs):
        """
        Initialize the object with the given keyword arguments.

        :param kwargs: keyword arguments
        :return: None
        """

        super().__init__(**kwargs)
        if self.enabled:
            self.client = "Yfinance"

    async def fetch(self, ticker=None):
        """
        Fetches the information for a given instrument from
        yahoo finance.

        Args:
            ticker_reference (str): The ticker symbol or
            reference of the instrument. Defaults to "MSFT".

        Returns:
            str: The formatted string containing the title
            and link of the latest news article for the instrument.
                 Returns None if there is no news available.
        """
        if ticker:
            logger.debug("Fetching news for {}", ticker)
            ticker = yf.Ticker(ticker)
            if news := ticker.news:
                title = news[0].get("title")
                link = news[0].get("link")
                return f"🗞️ <a href='{link}'>{title}</a>"

    async def monitor(self):
        """
        Asynchronously monitors the system and retrieves
        various data sources based on the configured settings.
        Cover Events, Feed, and Signal.

        Returns:
            str: A string containing the concatenated results
             of the retrieved data sources.
        """
        return await self.fetch(ticker=self.instrument)



import asyncio

from loguru import logger
from prettytable import PrettyTable
from tradingview_ta import TA_Handler

from ._client import Client


class TradingviewHandler(Client):
    """
    TradingviewHandler


    """

    def __init__(self, **kwargs):
        """
        Initialize the object with the given keyword arguments.

        :param kwargs: keyword arguments
        :return: None
        """

        super().__init__(**kwargs)
        logger.debug("Initializing TradingviewHandler")
        if self.enabled:
            self.client = "Tradingview"

    async def fetch_tv_data(self, asset_id, exchange, screener, interval):
        """
        Fetches from Trading View the analysis
        of a given asset from a specified exchange
        and screener at a specified interval.
        Utilizes asynchronous handling to improve efficiency.
        more info:
        https://github.com/AnalyzerREST/python-tradingview-ta

        Args:
            asset_id (str): The ID of the asset.
            exchange (str): The exchange on which the asset is traded.
            screener (str): The screener used for analysis.
            interval (str): The interval at which the analysis is performed.

        Returns:
            str: The recommendation based on the analysis.
            Can be one of the following:
                - 'BUY': "🔼"
                - 'STRONG_BUY': "⏫"
                - 'SELL': "🔽"
                - 'STRONG_SELL': "⏬"
                - Any other value: "▶️"
        """
        # logger.debug(f"Analysis for {asset_id} at {exchange} for {interval}.")
        recommendation_map = {
            "BUY": "🔼",
            "STRONG_BUY": "⏫",
            "SELL": "🔽",
            "STRONG_SELL": "⏬",
        }
        # try:
        handler = TA_Handler(
            symbol=asset_id, exchange=exchange, screener=screener, interval=interval
        )
        analysis = await asyncio.get_event_loop().run_in_executor(
            None, handler.get_analysis
        )
        return recommendation_map.get(analysis.summary["RECOMMENDATION"], "▶️")
        # except Exception as error:
        #     logger.warning(f"Error fetching analysis: {error}")
        #     return "Error"

    async def fetch(self, interval="4h"):
        """
        Fetches the signal for a given interval.

        Args:
            interval (str): The interval for which
            to fetch the signal. Defaults to "4h".

        Returns:
            str: The signal table as a string or HTML formatted
        """
        signals = []
        table = PrettyTable(header=False)
        logger.debug("Fetching signal for interval {}", interval)

        for asset in self.instrument:
            current_signal = await self.fetch_tv_data(
                asset_id=asset["id"],
                exchange=asset["exchange"],
                screener=asset["screener"],
                interval=interval,
            )
            if current_signal:
                signal_item = {
                    "symbol": asset["id"],
                    "interval": interval,
                    "signal": current_signal,
                }
                table.add_row([asset["id"], current_signal])
                signals.append(signal_item)
        return table.get_html_string() if self.format == "HTML" else table.get_string()

    async def monitor(self):
        """
        Asynchronously monitors the system and retrieves
        various data sources based on the configured settings.
        Cover Events, Feed, and Signal.

        Returns:
            str: A string containing the concatenated results
             of the retrieved data sources.
        """

        return await self.fetch()



# from ._client import Client


# class WebsocketHandler(Client):
#     """
#     Websocket client

#     """

#     def __init__(self, **kwargs):
#         """
#         Initialize the object with the given keyword arguments.

#         :param kwargs: keyword arguments
#         :return: None
#         """

#         super().__init__(**kwargs)
#         if self.enabled:
#             self.client = "websocket"
#             self.websocket_url = "wss://echo.websocket.org/"
